@@ -2388,3 +2388,44 @@ e_cal_backend_ews_get_datetime_with_zone (ETimezoneCache *timezone_cache,
 
 	return dt;
 }
+
+/* This is adapted copy of calendar_config_get_icaltimezone(), to avoid dependency
+   on the evolution code in the backend. */
+ICalTimezone *
+e_cal_backend_ews_get_configured_evolution_icaltimezone (void)
+{
+	GSettingsSchemaSource *schema_source;
+	ICalTimezone *zone = NULL;
+	gchar *location = NULL;
+
+	schema_source = g_settings_schema_source_get_default ();
+	if (schema_source) {
+		GSettingsSchema *schema;
+
+		schema = g_settings_schema_source_lookup (schema_source, "org.gnome.evolution.calendar", TRUE);
+		if (schema) {
+			GSettings *settings;
+
+			settings = g_settings_new ("org.gnome.evolution.calendar");
+
+			if (g_settings_get_boolean (settings, "use-system-timezone"))
+				location = e_cal_util_get_system_timezone_location ();
+			else
+				location = g_settings_get_string (settings, "timezone");
+
+			g_clear_object (&settings);
+			g_settings_schema_unref (schema);
+		}
+	}
+
+	if (!location)
+		location = e_cal_util_get_system_timezone_location ();
+
+	if (location) {
+		zone = i_cal_timezone_get_builtin_timezone (location);
+
+		g_free (location);
+	}
+
+	return zone;
+}
